import java.util.HashSet;
import java.util.Set;

/**
 * A NpdaVertex object models a single vertex in an npda.
 * This includes a label for the vertex, such as Q0, Q1, or
 * Q2, and a set of transitions from the vertex to other
 * vertices.
 * 
 * @author jmac
 * @date April 2010
 * 
 */
public class NpdaVertex {
	// the label of this vertex e.g. Q0, Q1, Q2
	private String label;

	// the set of transitions that are possible from this
	// vertex
	private Set<NpdaTransition> transitions;

	/**
	 * Create a new vertex with the given label. (e.g. Q0,
	 * Q1, Q2)
	 */
	public NpdaVertex(String label) {
		this.label = label;
		this.transitions = new HashSet<NpdaTransition>();
	}

	/**
	 * Add a new transition from this vertex to some other,
	 * destination vertex.
	 * 
	 * @param transition
	 *            the transition to be added
	 */
	public void addTransition(NpdaTransition transition) {
		assert (this.label.equals(transition.getSource()));
		transitions.add(transition);
	}

	/**
	 * Compute and return the set of all valid transitions
	 * that could be taken from this vertex, assuming we
	 * have just read the given readSymbol from the input
	 * string and popped the given popSymbol from the stack.
	 * 
	 * @param readSymbol
	 *            a symbol that has just being read from the
	 *            input string
	 * @param popSymbol
	 *            a symbol that has just been popped from
	 *            the stack
	 * @return the set of valid transitions will be given
	 *         readSymbol and popSymbol
	 */
	public Set<NpdaTransition> getTransitions(
			char readSymbol, char popSymbol) {
		Set<NpdaTransition> possibleTransitions = new HashSet<NpdaTransition>();
		for (NpdaTransition t : transitions) {
			if (t.isValid(readSymbol, popSymbol)) {
				possibleTransitions.add(t);
			}
		}
		return possibleTransitions;
	}

	/**
	 * return the label of this vertex
	 */
	public String getLabel() {
		return label;
	}

	/*
	 * (non-Javadoc) -- automatically generated by eclipse
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((label == null) ? 0 : label.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc) -- automatically generated by eclipse
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		NpdaVertex other = (NpdaVertex) obj;
		if (label == null) {
			if (other.label != null)
				return false;
		} else if (!label.equals(other.label))
			return false;
		return true;
	}

}
